<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All rights reserved.
 *
 * Author: Chandravadana Kandregula
 * Class Index
 * @module Pinpoint_DispatchTimer
 */

namespace Pinpoint\DispatchTimer\Controller\Json;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Pinpoint\DispatchTimer\Model\ShippingMessage;

class Index extends Action
{
    /**
     * @var JsonFactory
     */
    private $jsonResultFactory;
    /**
     * @var ShippingMessage
     */
    private $shippingMessage;

    /**
     * Index constructor.
     * @param Context $context
     * @param JsonFactory $jsonResultFactory
     * @param ShippingMessage $shippingMessage
     */
    public function __construct(Context $context, JsonFactory $jsonResultFactory, ShippingMessage $shippingMessage)
    {
        $this->jsonResultFactory = $jsonResultFactory;
        $this->shippingMessage = $shippingMessage;
        parent::__construct($context);
    }

    /**
     * This function handles incoming HTTP requests and processes them to generate a JSON response.
     * It retrieves 'sku' and 'message' parameters from the request, uses them to fetch shipping data
     * via the 'ShippingMessage' model, and returns the data as a JSON response with the appropriate headers.
     *
     * @return ResponseInterface|Json|ResultInterface
     */
    public function execute()
    {
        $sku = $this->getRequest()->getParam('sku') ? $this->getRequest()->getParam('sku') : '';
        $message = $this->getRequest()->getParam('message') ? $this->getRequest()->getParam('message') : '';
        $jsonResult = $this->jsonResultFactory->create();
        $jsonResult->setHeader('Content-Type', 'application/json');
        $jsonResult->setData($this->shippingMessage->getShippingData($sku, $message));

        return $jsonResult;
    }
}
