/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All rights reserved.
 *
 * Author: Chandravadana Kandregula
 * Code Release Version: 1.0.0
 *
 * @module Pinpoint_DispatchTimer;
 */

define([
        'jquery',
        'uiComponent',
        'ko'
    ], function ($, Component, ko) {
        'use strict';
        var self;
        return Component.extend({
            showCountdown: ko.observable(false),
            showStandard: ko.observable(false),
            dispatchMessage: ko.observable(''),
            shippingDateTime: ko.observable(''),
            standardMessage: ko.observable(''),
            showNone: ko.observable(false),
            dispatchTimer: null,
            initialize: function (config) {
                self = this;
                this._super();
                this.dispatchTimeMessage(config.url, config.sku, config.message);
            },

            dispatchTimeMessage: function (url, sku, message) {
                var requestData = {
                    sku: sku,
                    message: message
                };

                $.get(url, requestData, function (data) {
                    // If Module is Disabled or Product SKU is set as Not show any message on Config Settings
                    if (!data.status || data.nomessage_flag) {
                        self.showNone(false);
                        return;
                    }
                    // If Product Shipping Message is added or Standard Message Settings are updated in Config
                    if(data.standard_message){
                        self.standardMessage(data.standard_message);
                        self.showNone(true);
                        self.showStandard(true);
                        self.showCountdown(false);
                        return;
                    }

                    // Default Dispatch Timer message
                    self.dispatchTimer = data.dispatch_time;
                    self.shippingDateTime(countdownTimer(self.dispatchTimer));
                    setInterval(function () {
                        self.shippingDateTime(countdownTimer(self.dispatchTimer));
                    }, 1000);
                    self.showNone(true);
                    self.showCountdown(true);
                    self.dispatchMessage(data.message.replace('{day}', '<b>' + data.shipping_date_time + '</b>'));
                });

                function countdownTimer(dispatchDateTime) {
                    const difference = +new Date(dispatchDateTime) - +new Date();
                    let remaining = "Time's up!";
                    if (difference > 0) {
                        const parts = {
                            days: Math.floor(difference / (1000 * 60 * 60 * 24)),
                            hrs: Math.floor((difference / (1000 * 60 * 60)) % 24),
                            mins: Math.floor((difference / 1000 / 60) % 60),
                            secs: Math.floor((difference / 1000) % 60)
                        };
                        var label = '';
                        remaining = Object.keys(parts)
                            .map(part => {
                                if (!parts[part]) return;
                                if (`${parts[part]}` < 2) {
                                    label = (`${parts[part]} ${part}`).slice(0, -1);
                                } else {
                                    label = (`${parts[part]} ${part}`)
                                }
                                return `${label}`
                            })
                            .join(" ");
                    }
                    return remaining;
                }
            },
        });
    }
);
